class CorneliusException(Exception):
    def __init__(self, message: str = None):
        message = 'An fatal error has occurred' if message is None else message
        super().__init__(message)


class InvalidButtonException(CorneliusException):
    def __init__(self, message: str = None):
        message = 'The button is not valid' if message is None else message
        super().__init__(message)


class InvalidKeyException(CorneliusException):
    def __init__(self, message: str = None):
        message = 'The key is not valid' if message is None else message
        super().__init__(message)



from ctypes import windll, wintypes, byref
from .exceptions import InvalidButtonException

"""
    Cornelius cursor implementation for Windows.
    Based on the windows API and ctypes.
    Docstrings format is reStructuredText.
    Copyright (C) 2022 Manuel Cabral
    Licensed under the GNU General Public License v3.0
"""


class Cursor:
    __set = windll.user32.SetCursorPos
    __get = windll.user32.GetCursorPos
    __event = windll.user32.mouse_event
    __state = windll.user32.GetKeyState

    __dwFlags = {
        1: [0x2, 0x4],  # [Left down, Left up]
        2: [0x8, 0x10],  # [Right down, Right up]
    }

    RIGHT_BUTTON = 0x1
    LEFT_BUTTON = 0x2

    def __check_button(self, button) -> None:
        """
            Check if the button is valid.

            :param button: button to check
            :return: None
            :raises InvalidButtonException: if button is not int or not valid
        """
        if not isinstance(button, int):
            raise InvalidButtonException('button must be int')
        if not button in self.__dwFlags:
            raise InvalidButtonException(
                f'button must be one of {self.__dwFlags}')

    def set(self, pos_x: int, pos_y: int) -> None:
        """
            Set the cursor position.

            :param pos_x: x position
            :param pos_y: y position
            :return: None
            :raises TypeError: if pos_x or pos_y is not int
        """
        if not isinstance(pos_x, int) or not isinstance(pos_y, int):
            raise TypeError("x and y must be int")
        self.__set(pos_x, pos_y)

    def get(self) -> (int, int):
        """
            Get the cursor position.

            :return: (x, y)
        """
        point = wintypes.POINT()
        if self.__get(byref(point)):
            return point.x, point.y, point
        return 0, 0

    def press(self, button: int, **kwargs) -> None:
        """
            Press a button.

            :param button: button to press
            :param kwargs:
                - repeat: number of times to press
            :return: None
            :raises InvalidButtonException: if button is not int or not valid
        """
        self.__check_button(button)
        repeat = kwargs.get("repeat", 1)
        for _ in range(repeat):
            self.__event(self.__dwFlags[button][0], 0, 0, 0, 0)
            self.__event(self.__dwFlags[button][1], 0, 0, 0, 0)

    def is_pressed(self, button: int) -> bool:
        """
            Check if a button is pressed.

            :param button: button to check
            :return: True if pressed, False otherwise
            :raises InvalidButtonException: if button is not int or not valid
        """
        self.__check_button(button)
        return self.__state(button) & 0x80 == 0x80



from ctypes import windll
from .exceptions import InvalidKeyException

"""
    Cornelius keyboard implementation for Windows.
    Based on the windows API and ctypes.
    Docstrings format is reStructuredText.
    Copyright (C) 2022 Manuel Cabral
    Licensed under the GNU General Public License v3.0
"""


class Keyboard:
    __event = windll.user32.keybd_event
    __state = windll.user32.GetKeyState

    __dwFlags = {
        1: 0,       # Event key down
        2: 0x0002,  # Event key up
    }

    def __check_key(self, key) -> None:
        """
            Check if the key is valid.

            :param key: key to check
            :return: None
            :raises InvalidKeyException: if key is not str or int
        """
        if not isinstance(key, (str, int)):
            raise InvalidKeyException('key must be str or int')

    def __to_int(self, key: str) -> int:
        """
            Convert a key to its integer representation.

            :param key: key to convert
            :return: integer representation of the key
            :raises TypeError: if key is not str
        """
        if not isinstance(key, str):
            raise TypeError("key must be str")
        return ord(key.upper())

    def is_pressed(self, key) -> bool:
        """
            Check if a key is pressed.

            :param key: key to check
            :return: True if pressed, False otherwise
            :raises TypeError: if key is not str or int
        """
        self.__check_key(key)
        if isinstance(key, str):
            key = self.__to_int(key)
        return self.__state(key) & 0x80 == 0x80

    def press(self, key) -> None:
        """
            Press a key.

            :param key: key to press
            :return: None
            :raises InvalidKeyException: if key is not str or int
        """
        self.__check_key(key)
        if isinstance(key, str):
            key = self.__to_int(key)
        self.__event(key, 0, self.__dwFlags[1], 0)

    def release(self, key) -> None:
        """
            Release a key.

            :param key: key to release
            :return: None
            :raises InvalidKeyException: if key is not str or int
        """
        self.__check_key(key)
        if isinstance(key, str):
            key = self.__to_int(key)
        self.__event(key, 0, self.__dwFlags[2], 0)



from setuptools import setup, find_packages

with open('README.md', 'r') as file:
    long_description = file.read()

setup(
    name='cornelius',
    version='0.0.1',
    license='GPLv3',
    description='Python mouse and keyboard input control in a simple way.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Manuel Cabral',
    author_email='cabral.manuel@yandex.com',
    url='https://github.com/manucabral/cornelius',
    keywords=['mouseinput', 'keyboardinput', 'input', 'win32', 'inputcontrol'],
    packages=find_packages(),
    python_requires='>= 3.2',
    classifiers=[
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'Programming Language :: Python :: 3',
        'Operating System :: Microsoft :: Windows',
    ],
)
